
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

local coroutine = coroutine
local GetGuildRosterInfo = GetGuildRosterInfo
local GetNumGuildMembers = GetNumGuildMembers

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local L = LibStub:GetLibrary("SixSocial-1.0", true)
if not L then return end

local Enum = L.Enum
local falsy = L.falsy
local FormatUnit = L.FormatUnit
local merge = L.merge

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

local function GetGuildMemberStatus(online, isAway, isMobile)
	if not online then
		return Enum.Status.Offline
	elseif isMobile then
		return Enum.Status.Mobile
	elseif isAway == 1 then
		return Enum.Status.Away
	elseif isAway == 2 then
		return Enum.Status.Busy
	else
		return Enum.Status.Online
	end
end

-------------------------------------------------------------------------------
-- Library
-------------------------------------------------------------------------------

function L:GetGuildMember(index)
	local name, rank, rankIndex, level, className, zone, note, _, online, isAway, _, _, _, isMobile, _, _, guid = GetGuildRosterInfo(index)
	if (not name) then return end
	local status = GetGuildMemberStatus(online, isAway, isMobile)
	local unit = FormatUnit(name)

	return merge(L.CLUB_MEMBER, {
		[Enum.ClubMember.ClassName] = className,
		[Enum.ClubMember.Guid] = guid,
		[Enum.ClubMember.Id] = index,
		[Enum.ClubMember.Level] = level,
		[Enum.ClubMember.Name] = unit,
		[Enum.ClubMember.Note] = falsy(note),
		[Enum.ClubMember.Rank] = rank,
		[Enum.ClubMember.RankOrder] = rankIndex,
		[Enum.ClubMember.Status] = status,
		[Enum.ClubMember.StatusOrder] = Enum.StatusOrder[status],
		[Enum.ClubMember.Unit] = unit,
		[Enum.ClubMember.Zone] = zone,
	})
end

function L:IterateGuildMembers()
	return coroutine.wrap(function()
		for index = 1, GetNumGuildMembers() do
			local guildMember = self:GetGuildMember(index)

			if guildMember then
				coroutine.yield(guildMember)
			end
		end
	end)
end
